# -*- coding: utf-8 -*-
"""
Created on Mon Apr  8 12:47:03 2024

@author: matth
"""


import serial
import time
import sys
import matplotlib.pyplot as plt
import numpy as np
import h5py
from glob import glob

#sys.path.append('C:\\Users\\lmdealmeidanie\\surfdrive\\Code\\MAXRFScanner')
import amptek as amp
import h5_routines_dff as h5access

'''A Gcode line usually looks something like G1 X20 Y30 Z80 F1000 /r/n
The first part (G1 in this case), denotes the kind of command this is. G-commands tend to be fundamental commands. We only really use G0 and G1 lines in most cases, both of which are linear moves. G0 is always a max speed move, and G1 is a linear movement. M-commands are more specialized functions.
the X Y and Z parts describe the final position of the movement. If no argument is given to any of those three dimensions, the system assumes those dimensions stay constant.
the F argument describes the speed of the G1 movement, which is given in mm/minute.
'''

def XYZsafetycheck(X=None,Y=None,Z=None):
    #This function will check if a certain XYZ position is within the safe work area of the MA-XRF scanner and will raise a ValueError if not
    if X!=None and (X>290 or X<80):
        raise ValueError('X value outside of safe work area')
    if Y!=None and (Y>347 or Y<10):
        raise ValueError('Y value outside of safe work area')
    if Z!=None and (Z>334 or Z<85):
        raise ValueError('Z value outside of safe work area')
    return



class printer_master_class():
    def __init__(self):
        pass
        self.is_homed=False
        self.scanner=None


class printer_motor_class():
    def __init__(self, ident = None,SN=None):
        self.ident = ident
        self.master=SN[0]
        self.axis=SN[1]
        self.vel=1000 #This is velocity in mm per minute. Rest of the software uses mm per second! 
        self.name='Printer motor'+self.axis


    def connect_motor(self):
        if self.master.scanner is None:
            try:
                self.master.scanner=serial.Serial('COM5',baudrate=115200)
            except:
                return False
        return True
        pass
    def init_motor(self):
        #No action as it is part in general startup for this motor
        pass
    def reference_motor(self):
        if not self.master.is_homed:
            #Connect to the positioning system
#            self.master.scanner=serial.Serial('COM7',baudrate=115200)
            self.master.scanner.reset_input_buffer()
            #Home the positioning system
            self.master.scanner.write(("G28\r\n").encode())
            #This reads the response of the scanner
            response = self.master.scanner.readline()
            print(response.decode()) #This should print out 'leveling OFF'
            print("This should print out 'leveling OFF'")
            response = self.master.scanner.readline()
            print(response.decode()) #This should print out 'X:-19.00 Y:347.00 Z:334.00 E:0.00 Count X:0 Y:142800 Z:133600 B:0'
            print("This should print out 'X:-19.00 Y:347.00 Z:334.00 E:0.00 Count X:0 Y:142800 Z:133600 B:0'")
            response = self.master.scanner.readline()
            print(response.decode()) #This should print out 'ok'
            print("This should print out 'ok'")
            #Set absolute positioning (G90 command)
            self.master.scanner.write(("G90\r\n").encode())
            response = self.master.scanner.readline()
            print(response.decode()) #This should print out 'ok'
            print("This should print out 'ok'")
            #Wait for homing to finish
            self.master.scanner.write(("M400\r\n").encode())
            response = self.master.scanner.readline()
            print(response.decode()) #This should print out 'ok'
            print("This should print out 'ok'")
            #Move to initial position (just a starting position which moves the scan head to the safe operation area)
            #self.master.scanner.write(("G1 X140 Y330 Z320 F1000\r\n").encode())
            self.master.scanner.write(("G1 X140 Z320 F1000\r\n").encode())
            response = self.master.scanner.readline()
            print(response.decode()) #This should print out 'ok'
            print("This should print out 'ok'")
            self.master.scanner.reset_input_buffer()
            #Send a command for the system to wait until all current commands are fulfilled before continuing
            self.master.scanner.write(("M400\r\n").encode())
            response = self.master.scanner.readline()
            print(response.decode()) #This should print out 'ok'
            print("This should print out 'ok'")
            #Check position
            self.master.scanner.write(("M114\r\n").encode())
            response = self.master.scanner.readline()
            print(response.decode()) #This should print out 'X:140.00 Y:330.00 Z:320.00 E:0.00 Count X:0 Y:142800 Z:133600 B:0'            
            print("This should print out 'X:140.00 Y:330.00 Z:320.00 E:0.00 Count X:0 Y:142800 Z:133600 B:0' ")
            
            self.master.is_homed=True


    def amov(self,pos,retry=0):
        try:
            gcode='G1'
            debug=False        
            X,Y,Z=None,None,None
            if self.axis == 'X':
                gcode=gcode+' X'+str(float(pos))
                X=pos
            if self.axis == 'Y':
                gcode=gcode+' Y'+str(float(pos))
                Y=pos
            if self.axis == 'Z':
                gcode=gcode+' Z'+str(float(pos))
                Z=pos
            XYZsafetycheck(X,Y,Z)
    
            gcode=gcode+' F'+str(float(self.vel))+' \r\n'
            if debug:
                print(gcode)
            #Reset the input buffer of the positioning system
            self.master.scanner.reset_input_buffer()
            #Send the encoded gcode line to the positioning system
            self.master.scanner.write((gcode).encode())
            #Read the response of the positioning system
            response = self.master.scanner.readline()
            if debug:
                print(response.decode())
            #Send a command for the system to wait until all current commands are fulfilled before continuing
            self.master.scanner.write(("M400\r\n").encode())
            response = self.master.scanner.readline()
            if debug:
                print(response.decode())
            #Get the current position from the positioning system
            self.master.scanner.write(("M114\r\n").encode())
            response = self.master.scanner.readline()
            if debug:
                print(response.decode())
            if debug:
                print('This should have been the position')
    
            self.position=pos
        except:
            if retry < 3:
                self.amov(pos,retry+1)
            else:
                print('Moving detector not working'+self.axis+str(pos))
    
    


    def amov_nowait(self,pos,retry=0):
        try:
            #This is for safety, so that there is no problem with giving running motors orders
            #Send a command for the system to wait until all current commands are fulfilled before continuing
            self.master.scanner.write(("M400\r\n").encode())
            response = self.master.scanner.readline()
            print(response.decode())
    
    
            gcode='G1'
            
            X,Y,Z=None,None,None
            if self.axis == 'X':
                gcode=gcode+' X'+str(float(pos))
                X=pos
            if self.axis == 'Y':
                gcode=gcode+' Y'+str(float(pos))
                Y=pos
            if self.axis == 'Y':
                gcode=gcode+' Z'+str(float(pos))
                Z=pos
            XYZsafetycheck(X,Y,Z)
    
            gcode=gcode+' F'+str(float(self.vel))+' \r\n'
            #Reset the input buffer of the positioning system
            self.master.scanner.reset_input_buffer()
            #Send the encoded gcode line to the positioning system
            self.master.scanner.write((gcode).encode())
            #Read the response of the positioning system
            response = self.master.scanner.readline()
            print(response.decode())
            #Get the current position from the positioning system
            self.master.scanner.write(("M114\r\n").encode())
            response = self.master.scanner.readline()
            print(response.decode())
            print('This should have been the position')
    
            self.position=pos
        except:
            if retry < 3:
                self.amov(pos,retry+1)
            else:
                print('Moving detector not working'+self.axis+str(pos))
    
    def pos(self):
        debug=True
        self.master.scanner.reset_input_buffer()
        #Check position
        try:
            self.master.scanner.write(("M114\r\n").encode())
        except:
            print('I timed out while reading the position.')
            return self.position
        response = self.master.scanner.readline().decode()
        if debug:
            print(response)
        response_cut=response[response.find(self.axis)+2:response[response.find(self.axis)+2:].find(' ')+response.find(self.axis)+2]
        if debug:
            print(response_cut)
        try:
            self.position=np.float32(response_cut)
        except:
            pass
        if debug:
            print(self.position)
        return self.position
        
    def gvel(self):
        return self.vel/60.
    def svel(self,vel=20):
        self.vel=vel*60.

        
    
    def shutdown(self):
        print('The printer needs no shutdown')
        pass




# def moveandacc(X=None,Y=None,Z=None,mtime=1):
#     #This function will command the MA-XRF scanner to move to a certain XYZ position, and do a measurement with a length of mtime (in seconds)
#     #First do a positional safety check
#     XYZsafetycheck(X,Y,Z)
#     #Initiate the Gcode command. G1 (linear movement at set speed) is always used for movement
#     gcode='G1'
#     #Initiate the position segment of the gcode line
#     pos=''
#     #Add X movement
#     if X!=None:
#         gcode=gcode+' X'+str(float(X))
#         Xpos='{:06.2f}'.format(X)
#         pos=pos+'X'+Xpos
#     #Add Y movement
#     if Y!=None:
#         gcode=gcode+' Y'+str(float(Y))
#         Ypos='{:06.2f}'.format(Y)
#         pos=pos+'Y'+Ypos
#     #Add Z movement
#     if Z!=None:
#         gcode=gcode+' Z'+str(float(Z))
#         Zpos='{:06.2f}'.format(Z)
#         pos=pos+'Z'+Zpos
#     #Add termination of G1 line (including F speed, set in millimeters per minute, currently set at 1000)
#     gcode=gcode+'F1000 \r\n'
#     #Reset the input buffer of the positioning system
#     scanner.reset_input_buffer()
#     #Send the encoded gcode line to the positioning system
#     scanner.write((gcode).encode())
#     #Read the response of the positioning system
#     response = scanner.readline()
#     print(response.decode())
#     #Send a command for the system to wait until all current commands are fulfilled before continuing
#     scanner.write(("M400\r\n").encode())
#     response = scanner.readline()
#     print(response.decode())
#     #Get the current position from the positioning system
#     scanner.write(("M114\r\n").encode())
#     response = scanner.readline()
#     print(response.decode())
#     #Perform the measurement
#     acc,stat=detector.acc_spec(mtime=mtime)
#     #Return the measured data
#     return acc,stat

# def ScanPoints(Points,mtime=1):
#     #This function will do a series of point measurements based on the input Points list. The points should be described as a relative position from a certain Starting position (StartX, StartY, StartZ), preferably at the bottom left of the object.
#     #First define the number of points in the measurement series
#     nPoints=Points.shape[0]
#     #Find the number of dimensions used in the measurement
#     dims=Points.shape[1]
#     #Initiate results list
#     results=[]
#     #If only 2 dimensions, it is assumed they are X and Y
#     if dims==2:
#         #Loop over all points
#         for lp1 in range(0,nPoints):
#             #Retrieve X and Y positions from the list and ignore Z
#             X=Points[lp1,0]+StartX
#             Y=Points[lp1,1]+StartY
#             Z=None
#             #Move to location and run the measurement
#             acc,stat=moveandacc(X,Y,Z,mtime=mtime)
#             #Append results to the list
#             results.append((acc,stat))
#     elif dims==3:
#         #Loop over all points
#         for lp1 in range(0,nPoints):
#             #Retrieve X, Y and Z positions
#             X=Points[lp1,0]+StartX
#             Y=Points[lp1,1]+StartY
#             Z=Points[lp1,2]+StartZ
#             #Move to location and run the measurement
#             acc,stat=moveandacc(X,Y,Z,mtime=mtime)
#             #Append results to the list
#             results.append((acc,stat))
#     #If dims is not 2 or 3 then the points input must be structured wrongly
#     else:
#         raise ValueError('Measurement Points input is not formated correctly')
#     #Return resuts list    
#     return results

# def RasterScan(X1,X2,Y1,Y2,stepsize=1,mtime=0.1):
#     #This function will do 2D raster scan over a defined area, with a specified step size and measurement time per point
#     XYZsafetycheck(X1,Y1)
#     XYZsafetycheck(X2,Y2)
#     #Calculate the number of pixels in the measurement
#     Xpix=int(np.ceil((X2-X1)/stepsize))
#     Ypix=int(np.ceil((Y2-Y1)/stepsize))
#     npix=Xpix*Ypix
#     #Define speed in mm/sec (F argument/60)
#     speed=1000/60
#     #Calculate the total time estimate for the measurement
#     TotalTime=npix*(mtime+stepsize/speed)
#     #Provide the estimate to the used and request a go ahead
#     answer = input("Time estimate: "+str(int(np.floor(TotalTime/3600)))+ "h "+str(int(np.ceil((TotalTime%3600)/60)))+"m. Continue? [y/n]")
#     #If n, stop the measurement
#     if answer.lower() in ["n"]:
#         return
#     #If y, start the measurement
#     elif answer.lower() in ["y"]:
#         #Initiate data and status arrays
#         Data=np.zeros((Xpix,Ypix,2048))
#         Status=np.zeros((Xpix,Ypix,64))
#         #Start loop for Y direction
#         for lp1 in range(0,Ypix):
#             #Start loop for X direction
#             for lp2 in range(0,Xpix):
#                 #If y line is even, measure from left to right
#                 if (lp1%2)==0:
#                     Data[lp2,lp1,:],Status[lp1,lp2,:]=moveandacc(X1+lp2*stepsize,Y1+lp1*stepsize,mtime=mtime)
#                 #If y line is odd, measure from right to left
#                 else:
#                     Data[Xpix-lp2-1,lp1,:],Status[lp1,lp2,:]=moveandacc(X1+stepsize*(Xpix-lp2),Y1+lp1*stepsize,mtime=mtime)
#         #Print measurement
#         print('Finished measurement') 
#         return Data, Status

# #This is just useful for my PC setup, to check if im on my laptop or on my desktop
# test=glob('E:\\PhD\\Marker.txt')
# if glob('E:\\PhD\\Marker.txt')!=[]:
#     Surfdrive='E:\\PhD\\'
# elif glob('C:\\Users\\lmdealmeidanie\\surfdrive\\Marker.txt')!=[]:
#     Surfdrive='C:\\Users\\lmdealmeidanie\\surfdrive\\'

# #This documents the ranges I use for the safety limits. It is not exactly the maximum, I gave about 1cm ish on each end to make sure there are no collisions
# #Max Range X:80-290 (lots of danger!)  Y: 10-347 (no real danger)     Z: 85-334 (lots of danger!)






# #Initiate Scanner. THIS MUST BE DONE WHEN THE BOX IS OPEN!!!!!
# #OPEN THE BOX!!
# #IN THE NAME OF ALL THAT YOU HOLD DEAR PLEASE OPEN THE BOX BEFORE RUNNING THIS!!
# #Otherwise the homing of the positioning system will hit the measurement head onto the box

# #Connect to the detector
# detector=amp.Amptek_SDD()
# detector.connect()


#After this, the lead box can be closed, and afterwards the X-ray tube can be turned on through the amptek control software


