#Copyright (c) 2019, Matthias Alfeld
#All rights reserved.
#
#Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
#* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
#* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
#* The names of the contributors may not be used to endorse or promote products derived from this software without specific prior written permission.
#
#THIS SOFTWARE IS PROVIDED BY Matthias Alfeld ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Matthias Alfeld BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import h5py
import os
import numpy as np



datas=['data','raw','spectra','H','fitted_data']
labelss=['labels','ele','wavelength','wavelength [nm]']

def info_h5(ftr):
    if ftr is None:
        print('argument missing')
        return
    f=h5py.File(ftr,'r')
    keys=list(f.keys())
    dims=[]
    for k in keys:
        dims.append(f[k].shape)
    
    data_dict = {}
    for i in range(len(keys)):
        data_dict[keys[i]] = dims[i]
    
    return data_dict
    pass

def info_data_h5(ftr=None):
    if ftr is None:
        print('argument missing')
        return
    data_dict=info_h5(ftr)    
    for ds in datas:
        if ds in data_dict:
            return data_dict[ds]
    return None


def get_data_handle_h5(ftr=None):
    if ftr is None:
        print('argument missing')
        return
    f=h5py.File(ftr,'r')
    keys=list(f.keys())
    for ds in datas:
        if ds in keys:
            return f[ds]
    return None

    
def load_W(file):
    return load_h5(file,'W')

def load_H(file):
    return load_h5(file,'H')

def load_HW(file):
    HW=load_h5(file,('H','W'))
    return HW[0],HW[1]

def load_data_h5(file=None):
    if file is None:
        print('argument missing')
        return    
#    datas=['data','raw','spectra','H','fitted_data']
#    labelss=['labels','ele','wavelength','wavelength [nm]','H_labels']
    data,labels=None,None
    f=h5py.File(file,'r')
    for d in datas:
        if d in f:
            data=f[d][...]
            break
    for d in labelss:
        if d in f:
            labels=f[d][...]
            if d =='wavelength': #For removing the channel width column from ENVI
                if len(labels.shape) == 2:
                    labels=labels[0,:]
    if labels is None and not data is None:
        labels=[]
        for i in range(data.shape[2]):
            labels.append(i)
        labels=np.asarray(labels)
            
    return data, labels
    pass
#This is the standard reader that has a library for labels and data
    
def load_label_h5(ftr=None):
    if ftr is None:
        print('argument missing')
        return    
#    datas=['data','raw','spectra','H','fitted_data']
#    labelss=['labels','ele','wavelength','wavelength [nm]','H_labels']
    labels=None
    f=h5py.File(ftr,'r')
    for d in labelss:
        if d in f:
            labels=f[d][...]
            if d =='wavelength': #For removing the channel width column from ENVI
                if len(labels.shape) == 2:
                    labels=labels[0,:]
    f.close()            
    return labels



   
def load_h5(file,key):
    if file is None or key is None:
        print('argument missing')
        return
    f=h5py.File(file,'r')
    data=[]
    
    if type(key).__name__ != 'list' and type(key).__name__ != 'tuple':
        key=[key]

    for k in key:
        if k in f:
            ta=f[k][...]
            if len(ta.shape) == 0:
                ta=str(ta)
            data.append(ta)
#            if type(data[-1])==type(b'aaa') or type(data[-1]) == np.bytes_:
            if type(data[-1][0]).__name__ in ['bytes','bytes_']:
                data[-1]=[x.decode('unicode_escape') for x in data[-1]]
        else:
            print(k+' not found in '+file)
    f.close()    
    if len(key) == 1 and len(data) >0:
        data=data[0]
    return data


def load_data_h5_slice(file=None,sl=0,dim=2):
    if file is None:
        print('argument missing')
        return    
    #datas=['data','raw','spectra','H']
#    labelss=['labels','ele','wavelength','wavelength [nm]']
#    data,labels=None,None
    data=None
    f=h5py.File(file)
    for d in datas:
        if d in f:
            if dim == 0:
                data=f[d][sl,:,:]
            if dim == 1:
                data=f[d][:,sl,:]
            if dim == 2:
                data=f[d][:,:,sl]
            break
    return data
    pass
#here he is loading only a slice

def load_h5_slice(file=None, key=None, sl=0,dim=2):
    if file is None or key is None:
        print('argument missing in load_h5_slice')
        return
    no_close=False
    if type(file).__name__ == 'File':
        f=file
        no_close=True
    else:
        f=h5py.File(file,'a')
    if key in f:
        if dim == 0:
            data=f[key][:,:,sl]
        if dim == 1:
            data=f[key][:,sl,:]
        if dim == 2:
            data=f[key][sl,:,:]
    if not no_close:
        f.close()
    return data
    pass


def write_h5(file=None, keys=None, data=None):
    if file is None or keys is None or data is None:
        print('argument missing')
        return
    f=h5py.File(file,'a')
    for kk in list(f.keys()):
        del f[kk]
    add_h5(f,keys,data)
    f.close()
    print('data written to '+file)


def write_h5_slice(file=None,data=None,sl=0,shape=None,key=None):
    if file is None or key is None or data is None:
        print('argument missing in write_h5_slice')
        return
    no_close=False
    if type(file).__name__ == 'File':
        f=file
        no_close=True
        fname=os.path.basename(f.filename)
    else:
        f=h5py.File(file,'a')
        fname=f

    if not key in f:
        if not shape is None:
            f.create_dataset(key,shape)
            f[key][:,:,sl]=data
    else:
        f[key][:,:,sl]=data
    if not no_close:
        f.close()
    print(str(sl)+' slice was written to '+fname)

    pass

def add_h5(file=None, keys=None, data=None):
    if file is None or keys is None or data is None:
        print('argument missing')
        return
    no_close=False
    if type(file).__name__ == 'File' :
        f=file
        no_close=True
        fname=os.path.basename(f.filename)
    else:
        f=h5py.File(file,'a')
        fname=file
    if not type(keys).__name__ == 'list' and not type(keys).__name__ == 'tuple':
        keys=[keys,]
    if len(keys) == 1:# and (not type(data).__name__ == 'list' and not type(data).__name__ == 'tuple'):
        data=[data,]
    if len(keys) == len(data):
        for k,d in zip(keys,data):
            print(fname+' is updated with '+k+' '+type(d).__name__)
            if k in f:
                del f[k]
            if type(d[0]).__name__ == 'str' or type(d[0]).__name__ == '_str':
                d = [n.encode("ascii", "ignore") for n in d]
            f[k]=d            
    else:
        print('Data and keys have different length for',keys)
#    else:
#        if keys in f:
#            del f[keys]
#        if type(data[0]).__name__ == 'str' or type(data[0]).__name__ == 'str_':
#            data = [n.encode("ascii", "ignore") for n in data]
#            print('But I did as ordered!')
#        print(keys, type(data[0]).__name__)
#        f[keys]=data 
    if not no_close:
        f.close()

doit=False
if __name__ == '__main__' and doit:  

    print('General test of H5 routines')
    data_file='D:\\data\\Data_Handler_test\\auto_test\\hsi_merged.h5'
    res_file='D:\\data\\Data_Handler_test\\auto_test\\hsi_merged_result.h5'
    test_write='D:\\data\\Data_Handler_test\\auto_test\\hsi_merged_write_test.h5'
    print(info_h5(data_file))
    print(info_h5(res_file))
    W=load_W(res_file)
    print(W.shape)
    H=load_H(res_file)
    print(H.shape)    
    H,W=load_HW(res_file)
    print(H.shape,W.shape)    
    data,labels=load_data_h5(data_file)
    print(data.shape)
    print(labels)
    data=load_data_h5_slice(data_file,10)
    print(data.shape)
    labels=load_h5(res_file,'labels')
    print(labels)
    data=load_h5_slice(res_file,'H',0)
    print(data.shape)
    H,W,labels=load_h5(res_file,('H','W','labels'))
    write_h5(test_write,('H','W','labels'),(H,W,labels))
    print(info_h5(res_file))
    print(info_h5(test_write))
    add_h5(test_write,('dada','dudu'),(np.arange(0,1000,10),np.arange(0,5000,25)))
    print(info_h5(test_write))
    
    f1=h5py.File(data_file,'r')
    f2=h5py.File(test_write,'a')
    for i in range(info_h5(data_file)['spectra'][2]):
        datas=load_h5_slice(f1,'spectra',i)
        shape=info_h5(data_file)['spectra']
        key='key'
        write_h5_slice(f2,datas,i,shape,key)
    f1.close()
    f2.close()
    info_h5(res_file)
    
    


#    write_h5_slice
    