# -*- coding: utf-8 -*-
"""
Created on Fri May 22 17:32:36 2015

@author: Matthias
"""
import ctypes
import time
#import pipython.picontrollerlib.gcshelper.gcscommands as PIWrap
import thorlabs_wrapper as ThorWrap
import numpy as np
import matplotlib.pyplot as plt


class PI_Motor():
    #Definition of functions
    def __init__(self, ident = None, SN=None):
        self.SN='0123456789'
        return
#        self.ident = ident
#        self.dll=PIWrap.GCSCommandsWrapper
#        self.mem=PIWrap.GCSCommands()
#        if SN == None:
#            self.SN='0115500164'
#            self.SN=self.dll.EnumerateUSB(self.mem)[0]
#        else:
#            self.SN=SN

    def list_device(self):
        return self.dll.EnumerateUSB(self.mem)[0]


    def connect_motor(self):
#        print self.SN
        try:
            self.dll.ConnectUSB(self.mem,self.SN)
        except:
            print("Could not connect to "+self.SN)
            return False
        buff=''
        buff=self.dll.qIDN(self.mem,buff,255)
        print( 'Connected to '+buff)
        
        buff=''
        buff=self.dll.qVER(self.mem,buff,255)
        print( 'With parameters '+buff)
        return True
        
    def init_motor(self,stage='M-403.8PD'):
        self.name=stage
        buff=''
        buff=self.dll.qCST(self.mem,'1',buff,255)
        print( buff)
        td=buff
        td=td[td.find('=')+1:td.find('\n')]
        print( td)
        if td != 'DEFAULT_STAGE':
            print( 'Connected to '+td)
            return
        st=self.dll.CST(self.mem,'1',stage)
        buff=''
        buff=self.dll.qCST(self.mem,'1',buff,255)
        td=buff
        td=td[td.find('=')+1:td.find('\n')]
        if st == 0:
            print('Motor set to '+td+' but was meant to be '+stage)
        #This is the moment to switch on the 
    
    def reference_motor(self):
        isref='1'
        isref=self.dll.qSVO(self.mem,'1',isref)
        if isref[0]:
            print( 'Stage is referenced')
        else:
            self.dll.SVO(self.mem,'1','1')
            try:
                self.dll.FRF(self.mem,'1')  # is reference switch
        #        st=self.dll.FPL(self.mem,'1') # is positive end switch
        #        st=self.dll.FNL(self.mem,'1') # is negative end switch
                print( 'Axis properly referenced')
            except:
                print( 'Reference failed')
    
    def simple_scan(self):
        if self.ID == None:
            return
        startpos=ctypes.c_double(0.)
        self.dll.PI_qPOS(ctypes.c_int(self.ID),ctypes.c_char_p('1'),ctypes.byref(startpos))    
        for i in range(10):
            self.dll.PI_MOV(ctypes.c_int(self.ID),ctypes.c_char_p('1'),ctypes.byref(ctypes.c_double(startpos.value+i)))
            ismoving=ctypes.c_bool(True)
            self.dll.PI_IsMoving(ctypes.c_int(self.ID),ctypes.c_char_p('1'),ctypes.byref(ismoving))
            while ismoving.value:
                time.sleep(.1)
                self.dll.PI_IsMoving(ctypes.c_int(self.ID),ctypes.c_char_p('1'),ctypes.byref(ismoving))
            spot=ctypes.c_double(0.)
            self.dll.PI_qPOS(ctypes.c_int(self.ID),ctypes.c_char_p('1'),ctypes.byref(spot))    
            print( 'Measuring at '+str(spot.value))
            time.sleep(1)
    
    def amov(self,dest):
        ismoving=0
        self.dll.MOV(self.mem,'1',[dest])
        ismoving=self.dll.IsMoving(self.mem,'1',[ismoving])
        while ismoving[0]:
            time.sleep(.01)
            ismoving=self.dll.IsMoving(self.mem,'1',[ismoving])

    def amov_nowait(self,dest):
        self.dll.MOV(self.mem,'1',[dest])


    def svel(self,vel):
        ismoving=0
        ismoving=self.dll.IsMoving(self.mem,'1',[ismoving])
        if not ismoving[0]:
            self.dll.VEL(self.mem,'1',[vel])

    def gvel(self):
        pos=0.
        pos=self.dll.qVEL(self.mem,'1',[pos])
        return pos[0]


    def pos(self):
        pos=0.
        try:
            pos=self.dll.qPOS(self.mem,'1',[pos])   
        except:
            pos=[0.]
            pass
        return pos[0]

    def dyn_test(self, start, stop, step, dwell_time):
        vel=step/dwell_time
        print( vel)
        self.dll.VEL(self.mem,'1',[vel])
        self.amov(start)
        npo=int((stop-start)/step)
        res=np.zeros((2,npo))
        sup=np.arange(0,npo*dwell_time,dwell_time)
        self.dll.MOV(self.mem,'1',[stop])
        st=time.time()        
        for i in range(npo):
            ts=sup[i]-time.time()+st
            if ts<0:
                ts=0
            tp=time.time()-st
            time.sleep(ts)
            pos=0.
            pos=self.dll.qPOS(self.mem,'1',[pos])   
            res[0,i]=time.time()-st
            res[1,i]=pos[0]
            print( sup[i], tp, ts,pos)
        return res
    
    def shutdown(self,ID=None):
        try:
            self.dll.CloseConnection(self.mem)
            print ('is shutdown')
        except:
            print( 'shutdown failed')
        

class Thorlabs_Motor():
    #Definition of functions
    def __init__(self, ident = None, SN=None):
        self.ident=self.name = ident
        self.dll=ThorWrap#._lib#load_library()
        if SN == None:
            SNlist=self.dll.list_available_devices()
            self.SN=SNlist[0][1]
        else:
            self.SN=SN

    def list_device(self):
        return self.dll.list_available_devices()

    def connect_motor(self):
        print( self.SN)
        try:
            self.mot=self.dll.Motor(self.SN)
            self.mot.enable()
            info=self.mot.hardware_info
            self.ident=self.name=info[0]
        except:
            print ("Could not connect to "+str(self.SN))
            return False
        print( 'Connected to '+info[0]+' '+info[1]+' '+info[2])
        return True
        
    def init_motor(self,stage='M-403.8PD'):
        pass


        
    def reference_motor(self):
        if self.mot.has_homing_been_completed:
            print( "stage is referenced")
        else:
            print( "stage is being homed")
            self.mot.move_home()
    
    def simple_scan(self):
        print( "Debugging tool for PI, not included for Thorlabs")
 
    def amov(self,dest):
        ismoving=True
        self.mot.move_to(dest)
        ismoving=self.mot.is_in_motion
        while ismoving:
            time.sleep(.01)
            ismoving=self.mot.is_in_motion

    def amov_nowait(self,dest):
        self.mot.move_to(dest)


    def svel(self,vel):
        ismoving=True
        ismoving=self.mot.is_in_motion
        accvel=self.mot.get_velocity_parameters()
        if not ismoving:
            self.mot.set_velocity_parameters(accvel[0],accvel[1],vel)

    def gvel(self):
        accvel=self.mot.get_velocity_parameters()
        return accvel[2]


    def pos(self):
        pos=0.
        try:
            pos=self.mot.position 
        except:
            pos=0.
            pass
        return pos

    def dyn_test(self, start, stop, step, dwell_time):
        vel=step/dwell_time
        print( vel)
        self.dll.VEL(self.mem,'1',[vel])
        self.amov(start)
        npo=int((stop-start)/step)
        res=np.zeros((2,npo))
        sup=np.arange(0,npo*dwell_time,dwell_time)
        self.dll.MOV(self.mem,'1',[stop])
        st=time.time()        
        for i in range(npo):
            ts=sup[i]-time.time()+st
            if ts<0:
                ts=0
            tp=time.time()-st
            time.sleep(ts)
            pos=0.
            pos=self.dll.qPOS(self.mem,'1',[pos])   
            res[0,i]=time.time()-st
            res[1,i]=pos[0]
            print( sup[i], tp, ts,pos)
        return res
    
    def shutdown(self,ID=None):
        try:
#            self.mot.disable()
            self.dll._cleanup()
            print( 'is shutdown')
        except:
            print( 'shutdown failed')
        


class Printer_Motor():
    #Definition of functions
    def __init__(self, ident = None, SN=None):
        self.ident=self.name = ident
        self.dll=ThorWrap#._lib#load_library()
        if SN == None:
            SNlist=self.dll.list_available_devices()
            self.SN=SNlist[0][1]
        else:
            self.SN=SN

    def list_device(self):
        return self.dll.list_available_devices()

    def connect_motor(self):
        print( self.SN)
        try:
            self.mot=self.dll.Motor(self.SN)
            self.mot.enable()
            info=self.mot.hardware_info
            self.ident=self.name=info[0]
        except:
            print ("Could not connect to "+str(self.SN))
            return False
        print( 'Connected to '+info[0]+' '+info[1]+' '+info[2])
        return True
        
    def init_motor(self,stage='M-403.8PD'):
        pass


        
    def reference_motor(self):
        if self.mot.has_homing_been_completed:
            print( "stage is referenced")
        else:
            print( "stage is being homed")
            self.mot.move_home()
    
    def simple_scan(self):
        print( "Debugging tool for PI, not included for Thorlabs")
 
    def amov(self,dest):
        ismoving=True
        self.mot.move_to(dest)
        ismoving=self.mot.is_in_motion
        while ismoving:
            time.sleep(.01)
            ismoving=self.mot.is_in_motion

    def amov_nowait(self,dest):
        self.mot.move_to(dest)


    def svel(self,vel):
        ismoving=True
        ismoving=self.mot.is_in_motion
        accvel=self.mot.get_velocity_parameters()
        if not ismoving:
            self.mot.set_velocity_parameters(accvel[0],accvel[1],vel)

    def gvel(self):
        accvel=self.mot.get_velocity_parameters()
        return accvel[2]


    def pos(self):
        pos=0.
        try:
            pos=self.mot.position 
        except:
            pos=0.
            pass
        return pos

    def dyn_test(self, start, stop, step, dwell_time):
        vel=step/dwell_time
        print( vel)
        self.dll.VEL(self.mem,'1',[vel])
        self.amov(start)
        npo=int((stop-start)/step)
        res=np.zeros((2,npo))
        sup=np.arange(0,npo*dwell_time,dwell_time)
        self.dll.MOV(self.mem,'1',[stop])
        st=time.time()        
        for i in range(npo):
            ts=sup[i]-time.time()+st
            if ts<0:
                ts=0
            tp=time.time()-st
            time.sleep(ts)
            pos=0.
            pos=self.dll.qPOS(self.mem,'1',[pos])   
            res[0,i]=time.time()-st
            res[1,i]=pos[0]
            print( sup[i], tp, ts,pos)
        return res
    
    def shutdown(self,ID=None):
        try:
#            self.mot.disable()
            self.dll._cleanup()
            print( 'is shutdown')
        except:
            print( 'shutdown failed')
        




if __name__ == u'__main__':
    print('We could do an automatic test, but even we are not that stupid')
